import 'dart:async';

import 'package:equatable/equatable.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:soliq_id/data/model/response/account_info_response.dart';
import 'package:soliq_id/data/model/response/create_session_response.dart';
import 'package:soliq_id/data/model/response/process_response.dart';
import 'package:soliq_id/data/repository/soliq_id_repository.dart';
import 'package:soliq_id/utils/utils.dart';

part 'soliq_id_event.dart';

part 'soliq_id_state.dart';

class SoliqIdBloc extends Bloc<SoliqIdEvent, SoliqIdState> {
  SoliqIdRepository repository;

  SoliqIdBloc(this.repository) : super(SoliqIdInitial()) {
    on<SoliqIdAccountInfoEvent>(_onAccountInfo);
    on<SoliqIdCreateSessionEvent>(_onCreateSession);
    on<SoliqIdProcessEvent>(_onSetProcess);
    on<SoliqIdCompareFaceEvent>(_onSetCompareFace);
  }

  FutureOr<void> _onAccountInfo(
      SoliqIdAccountInfoEvent event, Emitter<SoliqIdState> emit) async {
    try {
      final response = await repository.setAccountInfo(
        deviceId: await Utils.getDeviceId(),
        projectId: event.projectId,
        deviceName: (await Utils.getDeviceDetails()).toString(),
        factoryName: (await Utils.getDeviceDetails()).toString(),
        osType: Utils.osType(),
        pinfl: event.pinfl,
        osVersion:( await Utils.getDeviceDetails()).toString(),
        appVersion: ( await Utils.getDeviceDetails()).toString(),
        firebaseId: '',
        lang: event.lang,
      );
      emit(SoliqIdAccountInfoSuccessState(
        response: response,
      ));
    } on Object catch (e) {
      print(e.toString());
    }
  }


  FutureOr<void> _onCreateSession(
      SoliqIdCreateSessionEvent event, Emitter<SoliqIdState> emit) async {
    try {
      final response = await repository.createSession(
          deviceId: event.deviceId,
          projectId: event.projectId,
          deviceName: event.deviceName,
          factoryName: event.factoryName,
          osType: event.osType,
          pinfl: event.pinfl,
          osVersion: event.osVersion,
          appVersion: event.appVersion,
          firebaseId: event.firebaseId,
          lang: event.lang,
          fio: event.fio);
      emit(SoliqIdCreateSessionSuccessState(
        response: response,
      ));
    } on Object catch (e) {
      print(e.toString());
    }
  }


  FutureOr<void> _onSetProcess(
      SoliqIdProcessEvent event, Emitter<SoliqIdState> emit) async {
    try {
      final response = await repository.setProcess(
        deviceId: await Utils.getDeviceId(),

        id: event.id, asked: event.asked, processId: event.processId, date: '',
      );
      emit(SoliqIdProcessSuccessState(
        response: response,
      ));
    } on Object catch (e) {
      print(e.toString());
    }
  }

  FutureOr<void> _onSetCompareFace(
      SoliqIdCompareFaceEvent event, Emitter<SoliqIdState> emit) async {
    try {
      final response = await repository.compareFace(
        deviceId: await Utils.getDeviceId(),
        id: event.id, asked: event.asked, processId: event.processId, date: event.date,
      );
      emit(SoliqIdCompareFaceSuccessState(
        response: response,
      ));
    } on Object catch (e) {
      print(e.toString());
    }
  }
}
