import 'dart:async';
import 'dart:convert';
import 'dart:developer';
import 'dart:io';
import 'package:camera/camera.dart';
import 'package:soliq_id/data/model/angle_model.dart';
import 'package:soliq_id/data/model/create_session_model.dart';
import 'package:soliq_id/utils/color_utils.dart';
import 'package:soliq_id/utils/utils.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter_screen_wake/flutter_screen_wake.dart';
import 'package:google_mlkit_face_detection/google_mlkit_face_detection.dart';
import 'package:lottie/lottie.dart';
import 'package:soliq_id/views/camera_view.dart';
import 'package:soliq_id/utils/countdown.dart';
import 'package:soliq_id/utils/face_detector_painter.dart';

class CameraScreen extends StatefulWidget {
   final CreateSessionModel session;

  const CameraScreen({
    Key? key,
     required this.session,
  }) : super(key: key);

  @override
  State<CameraScreen> createState() => _CameraScreenState();
}

class _CameraScreenState extends State<CameraScreen> {
  final FaceDetector _faceDetector = FaceDetector(
    options: FaceDetectorOptions(
      enableContours: true,
      enableClassification: true,
    ),
  );

  bool _canProcess = true,
      _isBusy = false,
      _isValid = false,
      _isStartDetection = false;
  int leftPosition = 0,
      rightPosition = 0,
      straightPosition = 0,
      blinkPosition = 0,
      smilePosition = 0,
      durationCountDown = 5,
      _faceState = 0,
      tasksCount = 3,
      counter = 60,
      _brightnessState = 0,
      _timerBrightnessState = 3,
      faceCount = -1;
  double result = 0.0, stepSize = 0.0;
      late Timer _timer, _timerBrightness;
  final Utils _utils = Utils();
  FacePosition? _currentPosition;
  List<PositionModel> tasks = [];
  final List<Map<String, dynamic>> _clientDoneExercises = [];
  double _widthFace = 0.0,
      _heightFace = 0.0,
      turnTheHead = 0.0,
      _brightness = 0.1;
  double? _suggestAngleValue;
  CustomPaint? _customPaint;
  String? _text;
  String hint = "straight", control = "", stateCha = "";
  Rect? boundingBox;
  final List<FacePosition> _donePositions = [];
  InputImage? _brightnessInputImage;
  var idle = Idle.task;

  @override
  void initState() {
    super.initState();
    if(widget.session.lifeTime != null){
      counter = widget.session.lifeTime??60;
       counter = 10;
    } else {
      counter = 60;
    }
    startTimer();
    if(idle == Idle.brightness) startCheckBrightnessTimer();

    if(widget.session.tasks != null){
      tasksCount = widget.session.tasks!.length;
      tasks = _utils.tasksFromTasks(widget.session.tasks!);
    } else {
      tasks = _utils.randomTaskPosition(tasksCount);
    }
    stepSize = 100 / (tasks.length);

    FlutterScreenWake.brightness.then((value) => _brightness = value);
    FlutterScreenWake.setBrightness(1);
  }

  @override
  void dispose() {
    _canProcess = false;
    _faceDetector.close();
    FlutterScreenWake.setBrightness(_brightness);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      backgroundColor: AppColor.appColor,
      appBar: AppBar(
        elevation: 0,
        centerTitle: true,
        title: const Text(
          "Face ID",
          style: TextStyle(
              fontSize: 16, fontWeight: FontWeight.w500, color: Colors.black),
        ),
        backgroundColor: AppColor.appColor,
      ),
      body: SingleChildScrollView(
        child: Column(
          children: [
            SizedBox(
                child: Stack(
              alignment: AlignmentDirectional.center,
              children: [
                CameraView(
                  value: result,
                  suggestAngle: _suggestAngleValue,
                  isValid: _faceState > 0,
                  title: 'Face Detector',
                  customPaint: _customPaint,
                  text: _text,
                  onImage: (inputImage, imageString) {
                    processImage(inputImage, imageString);
                  },
                  onFinish: (filePath) async {
                    if (result > 95) {
                      if (filePath == '') return;
                      final bytes = await File(filePath).readAsBytes();
                      Navigator.of(context).pop({
                        "image": "data:image/png;base64,${base64Encode(bytes)}",
                        "history": _clientDoneExercises
                      });
                      _timer.cancel();
                    }
                  },
                  initialDirection: CameraLensDirection.front,
                ),
              ],
            )),
            const SizedBox(
              height: 20,
            ),
            Align(
                alignment: Alignment.center,
                child: Text(
                  "00 : ${counter.toString().length == 1 ? "0" : ""}$counter",
                  style: TextStyle(
                    fontSize: 16,
                    color: counter < 5 ? Colors.red : Colors.black,
                    fontWeight: FontWeight.w500,
                  ),
                )),
            Container(
              padding: const EdgeInsets.all(12),
              margin: const EdgeInsets.all(12),
              decoration: BoxDecoration(
                  color: faceCount != 1
                      ? AppColor.custom_redColor.withOpacity(0.2)
                      : const Color(0xFFF3F6F5),
                  borderRadius: BorderRadius.circular(12)),
              child: Column(
                children: [
                  Container(
                    margin: const EdgeInsets.symmetric(horizontal: 8),
                    padding: const EdgeInsets.symmetric(
                        horizontal: 40, vertical: 16),
                    decoration: BoxDecoration(
                      color: _isValid ? Colors.white : Colors.red,
                      borderRadius: const BorderRadius.all(Radius.circular(16)),
                    ),
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.center,
                      children: [
                        const SizedBox(
                          width: 8,
                        ),
                        Text(
                        idle == Idle.brightness ? "${"check_brightness"}: $_brightnessState" :  hint,
                          style: TextStyle(
                            color: !_isValid ? Colors.white : Colors.black,
                          ),
                        ),
                        const SizedBox(
                          width: 8,
                        )
                      ],
                    ),
                  ),
                  const SizedBox(
                    height: 18,
                  ),
                  if (tasks.isNotEmpty && idle != Idle.brightness)
                    Container(
                        width: 40,
                        height: 40,
                        margin: const EdgeInsets.symmetric(horizontal: 8),
                        child: Lottie.asset(tasks.first.image)),
                  if(idle == Idle.brightness) Text("$_timerBrightnessState", style: TextStyle(fontSize: 24 - _timerBrightnessState * 4, fontWeight: FontWeight.w600),),
                  const SizedBox(
                    height: 6,
                  ),
                  faceCountDetection(),
                  // Text(stateCha),
                ],
              ),
            )
          ],
        ),
      ),
    );
  }

  Future<void> processImage(InputImage inputImage, String imageString) async {
    if (!_canProcess) return;
    if (_isBusy) return;
    _isBusy = true;
    setState(() {
      _text = '';
    });
    final faces = await _faceDetector.processImage(inputImage);
    if (inputImage.inputImageData?.size != null &&
        inputImage.inputImageData?.imageRotation != null) {
      final painter = FaceDetectorPainter(
          faces,
          inputImage.inputImageData!.size,
          inputImage.inputImageData!.imageRotation);
      _customPaint = CustomPaint(painter: painter);
      faceCount = faces.length;

      if (faces.length > 1) {
        stateCha = "more_face_found";
        _utils.voiceAndVibrate(type: VoiceType.none);
        if (_isStartDetection) {
          _clientDoneExercises.add({"id": -20, "asked": tasks.first.id, "date": DateTime.now().toString()});
        }
      } else if (faces.length == 1) {
        _isStartDetection = true;
        final face = faces.first;

        boundingBox = face.boundingBox;
        _heightFace = boundingBox!.right - boundingBox!.left;
        _widthFace = boundingBox!.bottom - boundingBox!.top;

        var rightStateFace = AngleModel.MinFaceSize <= _heightFace &&
            _heightFace <= AngleModel.MaxFaceSize &&
            AngleModel.MinFaceSize <= _widthFace &&
            _widthFace <= AngleModel.MaxFaceSize;

        if (_faceState == 0 && rightStateFace) detectFace(face);

        if (AngleModel.MinFaceSize <= _heightFace ||
            AngleModel.MinFaceSize <= _widthFace) {
          hint = "face_away";
          _isValid = false;
          if (_isStartDetection) {
            _clientDoneExercises.add({"id": -30, "asked": tasks.first.id, "date": DateTime.now().toString()});
          }
        }
        if (AngleModel.MaxFaceSize >= _heightFace ||
            AngleModel.MaxFaceSize >= _widthFace) {
          hint = "face_closer";
          if (_isStartDetection) {
            _clientDoneExercises.add({"id": -40, "asked": tasks.first.id, "date": DateTime.now().toString()});
          }
          _isValid = false;
        }

        if (rightStateFace) {
          hint = "straight";
          _isValid = true;
          switch(idle){
            case Idle.brightness:
              _brightnessInputImage = inputImage;
              break;
            case Idle.task:
              checkRandomActive(face, inputImage, imageString);
              break;
            case Idle.capture:
              captureIdleAction(face, inputImage);
              break;
            case Idle.done:

              break;
          }
          // check360(face);
        }

        stateCha = """
        X o'qi bo'yicha ${_utils.currencyFormatter(face.headEulerAngleX, decimalDigit: 2)}
        Y o'qi bo'yicha ${_utils.currencyFormatter(face.headEulerAngleY, decimalDigit: 2)}
        Z o'qi bo'yicha ${_utils.currencyFormatter(face.headEulerAngleZ, decimalDigit: 2)}
        Kulish darajasi: ${_utils.currencyFormatter(face.smilingProbability, decimalDigit: 2)}
        Chap ko'z: ${_utils.currencyFormatter(face.leftEyeOpenProbability, decimalDigit: 2)} - O'ng ko'z ${_utils.currencyFormatter(face.rightEyeOpenProbability, decimalDigit: 2)}
        Face size: H: $_heightFace - W: $_widthFace
        boundingBox ${face.boundingBox}
        done positions: ${_utils.doneFacePositions(_donePositions)}
        Face done positions: $_donePositions
        """;
      } else {
        if (faceCount == 0 && _isStartDetection) {
          _clientDoneExercises.add({"id": -10, "asked": tasks.first.id, "date": DateTime.now().toString()});
          stateCha = "no_face_found";
          _utils.voiceAndVibrate(type: VoiceType.none);
        }
      }
    } else {
      String text = 'Faces found: ${faces.length}\n\n';
      if (faces.length > 1) {
        stateCha = "more_face_found";
      } else if (faces.length == 1) {
        final face = faces.first;
        stateCha =
        "Yuz holati:\n X o'qi bo'yicha ${face.headEulerAngleX}\n Y o'qi bo'yicha ${face.headEulerAngleY}\n Z o'qi bo'yicha ${face.headEulerAngleZ}\n Kulish darajasi: ${face.smilingProbability}\n Chap ko'z ochiqligi: ${face.leftEyeOpenProbability}\n O'ng ko'z ochiqligi ${face.rightEyeOpenProbability}";
      } else {
        stateCha = "no_face_found";
      }
      for (final face in faces) {
        text += 'face: ${face.boundingBox}\n\n';
      }
      _text = text;
      // TODO: set _customPaint to draw boundingRect on top of image
      _customPaint = null;
    }
    _isBusy = false;
    if (mounted) {
      setState(() {});
    }
  }

  Future<void> checkActive(
      Face face, InputImage inputImage, String imageString) async {
    if ((face.headEulerAngleY ?? 0) >= AngleModel.MinFaceTurningHorizontal &&
        (face.headEulerAngleY ?? 0) <= AngleModel.MaxFaceTurningHorizontal &&
        result == 0) {
      rightPosition = 20;
    }
    if ((face.headEulerAngleY ?? 0) <=
            -1 * AngleModel.MinFaceTurningHorizontal &&
        (face.headEulerAngleY ?? 0) >=
            -1 * AngleModel.MaxFaceTurningHorizontal &&
        result == 20) {
      leftPosition = 20;
    }
    if ((face.leftEyeOpenProbability ?? 0) <= 0.05 &&
        (face.rightEyeOpenProbability ?? 0) <= 0.05 &&
        result == 40) {
      blinkPosition = 20;
    }
    if ((face.smilingProbability ?? 0) >= 0.80 && result == 60) {
      smilePosition = 20;
    }

    if ((face.headEulerAngleY ?? 0) >= -5 &&
        (face.headEulerAngleY ?? 0) <= 10 &&
        (face.headEulerAngleX ?? 0) >= -5 &&
        (face.headEulerAngleX ?? 0) <= 10 &&
        (face.headEulerAngleZ ?? 0) >= -5 &&
        (face.headEulerAngleZ ?? 0) <= 5 &&
        (face.smilingProbability ?? 0) <= 0.10 &&
        (face.leftEyeOpenProbability ?? 0) >= 0.8 &&
        (face.rightEyeOpenProbability ?? 0) >= 0.8 &&
        result == 80) {
      straightPosition = 20;
    } else if (result == 100) {
      // print(inputImage.filePath);
      // print(inputImage.inputImageData);
      // print(inputImage.bytes);
      // String base64Image = "data:image/png;base64,$imageString";
      //
      // File imagefile = File.fromRawPath(inputImage.bytes!);
      // Uint8List imagebytes = await imagefile.readAsBytes();
      // String base64string = base64.encode(imagebytes);
      //
      // print("base64string: " + base64string);
      //
      // print("imageString: " + imageString);
      // print("base64Image: " + base64Image);

      // var bytes = await File(imageString).readAsBytes();
      // var base64Byte = base64Encode(bytes);
      //
      //
      // Navigator.of(context).pop(bytes);
    }
    hint = tasks.first.title;

    // result = leftPosition +
    //     rightPosition +
    //     blinkPosition +
    //     smilePosition +
    //     straightPosition; todo tuzatish kerak 360 ishlatish uchun
  }

  Future<void> checkRandomActive(
      Face face, InputImage inputImage, String imageString) async {
    var date = DateTime.now();
    var detect = checkFacePose(face, inputImage, date);
    if (tasks.length > 1) {
      if (detect == tasks.first.variant) {
        sendReport(_utils.getTaskIdFromVariant(detect), date);
        log("${tasks.length} - ${tasks.first.title}");
        hint = tasks.first.title;
        tasks.removeAt(0);
        _utils.voiceAndVibrate();
        result += stepSize;
        control = "";
      } else {
        if (detect != TaskVariantEnum.defaultPose) {
          control =  "error_accurred";
        }
      }
    } else {
      idle = Idle.capture;
    }

    if (tasks.isNotEmpty) hint = tasks.first.title;
  }

  void check360(Face face) {
    if ((face.headEulerAngleX ?? 0) > AngleModel.XTopInt &&
        (face.headEulerAngleX ?? 0) < AngleModel.XTopExt) {
      if ((face.headEulerAngleY ?? 0) < AngleModel.YRightInt) {
        addPosition(FacePosition.Top);
      } else {
        addPosition(FacePosition.TopRight);
      }
      if ((face.headEulerAngleY ?? 0) > AngleModel.YLeftInt) {
        addPosition(FacePosition.Top);
      } else {
        addPosition(FacePosition.TopLeft);
      }
    }
    if ((face.headEulerAngleX ?? 0) < AngleModel.XBottomInt &&
        (face.headEulerAngleX ?? 0) > AngleModel.XBottomExt) {
      if ((face.headEulerAngleY ?? 0) < AngleModel.YRightInt) {
        addPosition(FacePosition.Bottom);
      } else {
        addPosition(FacePosition.BottomRight);
      }
      if ((face.headEulerAngleY ?? 0) > AngleModel.YLeftInt) {
        addPosition(FacePosition.Bottom);
      } else {
        addPosition(FacePosition.BottomLeft);
      }
    }
    if ((face.headEulerAngleX ?? 0) > AngleModel.XBottomInt &&
        (face.headEulerAngleX ?? 0) < AngleModel.XTopInt) {
      if ((face.headEulerAngleY ?? 0) > AngleModel.YRightInt) {
        addPosition(FacePosition.Right);
      }
      if ((face.headEulerAngleY ?? 0) < AngleModel.YLeftInt) {
        addPosition(FacePosition.Left);
      }
    }
    if (_utils.compareFacePosition(_donePositions).length == 8) result = 100;
  }

  void addPosition(FacePosition position) {
    if (!_donePositions.contains(position)) {
      _donePositions.add(position);
    }

    _currentPosition = position;

    createCountDown(position);
  }

  void createCountDown(FacePosition position) {
    CountDown countDown = CountDown(Duration(seconds: durationCountDown));
    var stream = countDown.stream.listen(null);
    stream.onData((data) {
      if (kDebugMode) {
        print("position: $data");
      }
    });
    stream.onDone(() {
      setState(() {
        if (_currentPosition != position) _utils.suggestAngle(position);
      });
    });
  }

  void detectFace(Face face) {
    var stream =
        CountDown(const Duration(milliseconds: 500)).stream.listen(null);
    stream.onData((data) {
      if ((face.headEulerAngleX ?? 0) <= AngleModel.LookRight &&
          (face.headEulerAngleY ?? 0) <= AngleModel.LookRight &&
          (face.headEulerAngleX ?? 0) >= (-1 * AngleModel.LookRight) &&
          (face.headEulerAngleY ?? 0) >= (-1 * AngleModel.LookRight)) {
      } else {
        stream.cancel();
      }
    });
    stream.onDone(() {
      if ((face.headEulerAngleX ?? 0) <= AngleModel.LookRight &&
          (face.headEulerAngleY ?? 0) <= AngleModel.LookRight &&
          (face.headEulerAngleX ?? 0) >= (-1 * AngleModel.LookRight) &&
          (face.headEulerAngleY ?? 0) >= (-1 * AngleModel.LookRight)) {
        _faceState = 1;
      }
    });
  }

  TaskVariantEnum checkFacePose(Face face, InputImage inputImage, date) {
    if (Platform.isAndroid) {
      if ((face.headEulerAngleY ?? 0) >= AngleModel.MinFaceTurningHorizontal) {
        _clientDoneExercises.add({"id": 2, "asked": tasks.first.id, "date": date.toString()});
        return TaskVariantEnum.left;
      }
      if ((face.headEulerAngleY ?? 0) <=
          -1 * AngleModel.MinFaceTurningHorizontal) {
        _clientDoneExercises.add({"id": 4, "asked": tasks.first.id, "date": date.toString()});
        return TaskVariantEnum.right;
      }
    } else {
      if ((face.headEulerAngleY ?? 0) >= AngleModel.MinFaceTurningHorizontal) {
        _clientDoneExercises.add({"id": 4, "asked": tasks.first.id, "date": date.toString()});

        return TaskVariantEnum.right;
      }
      if ((face.headEulerAngleY ?? 0) <=
          -1 * AngleModel.MinFaceTurningHorizontal) {
        _clientDoneExercises.add({"id": 2, "asked": tasks.first.id, "date": date.toString()});
        return TaskVariantEnum.left;
      }
    }

    if ((face.headEulerAngleX ?? 0) >= AngleModel.MinFaceTurningVertical) {
      _clientDoneExercises.add({"id": 1, "asked": tasks.first.id, "date": date.toString()});
      return TaskVariantEnum.top;
    }
    if ((face.headEulerAngleX ?? 0) <= -1 * AngleModel.MinFaceTurningVertical) {
      _clientDoneExercises.add({"id": 3, "asked": tasks.first.id, "date": date.toString()});

      return TaskVariantEnum.down;
    }

    if ((face.leftEyeOpenProbability ?? 0) <= 0.05 &&
        (face.rightEyeOpenProbability ?? 0) <= 0.05) {
      _clientDoneExercises.add({"id": 5, "asked": tasks.first.id, "date": date.toString()});
      return TaskVariantEnum.blink;
    }
    if ((face.smilingProbability ?? 0) >= 0.80) {
      _clientDoneExercises.add({"id": 6, "asked": tasks.first.id, "date": date.toString()});
      return TaskVariantEnum.smile;
    }

    if ((face.headEulerAngleY ?? 0) >= -8 &&
        (face.headEulerAngleY ?? 0) <= 12 &&
        (face.headEulerAngleX ?? 0) >= -5 &&
        (face.headEulerAngleX ?? 0) <= 10 &&
        (face.headEulerAngleZ ?? 0) >= -5 &&
        (face.headEulerAngleZ ?? 0) <= 5 &&
        (face.smilingProbability ?? 0) <= 0.10 &&
        (face.leftEyeOpenProbability ?? 0) >= 0.8 &&
        (face.rightEyeOpenProbability ?? 0) >= 0.8) {
      _clientDoneExercises.add({"id": 7, "asked": tasks.first.id, "date": date.toString()});
      return TaskVariantEnum.straight;
    }
    return TaskVariantEnum.defaultPose;
  }

  faceCountDetection() {
    switch (faceCount) {
      case 0:
        return Text( "no_face_found");
      case -1:
      case 1:
        return const SizedBox();
    }
    if (faceCount > 1) return Text( "more_face_found");
  }

  Future<void> sendReport(int id, DateTime date) async {
    Map<String, dynamic> params = {};
    params["id"] = id;
    params["date"] = date.toString();

    // await ApiProvider().sendReport(context, params).then((value) async {
    //   if(!value.success) {
    //     if(value.code == 10) {
    //       Dialogs.showDialogOk(
    //       value.message ?? "Berilgan vaqt tugadi",
    //       context,
    //       onYesTap: (){
    //         idle = Idle.done;
    //         Navigator.pop(context);
    //       }
    //   );
    //     }
    //   }
    // }).catchError((onError) {
    //   Utils.onError(context, onError);
    // }).whenComplete(() {});
  }

  void startTimer() {
    _timer = Timer.periodic(const Duration(seconds: 1), (timer) {
      if (counter < 1) {
        timer.cancel();
        idle = Idle.done;
        Navigator.pop(context);
      } else {
        counter--;
      }

    });
  }

  void startCheckBrightnessTimer(){
    _timerBrightness = Timer.periodic(const Duration(seconds: 1), (timer) {
      var brightnessRate = Platform.isAndroid ? 60 : 100;
      if(_brightnessInputImage != null) _brightnessState = _utils.brightness(_brightnessInputImage!);
      _timerBrightnessState = 3 - timer.tick;
      log("timerBrightness: $_timerBrightnessState - timer.tick: ${timer.tick}");

      if (timer.tick == 3) {
        timer.cancel();
        _timerBrightness.cancel();
        if(_brightnessState < brightnessRate){
          // Dialogs.showDialogOk(
          //     getTranslated(context, "low_brightness"),
          //     context,
          //   onYesTap: (){
          //     idle = Idle.done;
          //     Navigator.pop(context);
          //   }
          // );
        } else {
          _utils.voiceAndVibrate(type: VoiceType.turn);
          idle = Idle.task;
        }
      }
    });
  }

  Future<void> captureIdleAction(Face face, InputImage inputImage) async {
    var maxRange = 7;
    if((face.headEulerAngleX ?? 0) <= -1 * maxRange){
      hint = "head_rise";
    } else if((face.headEulerAngleX ?? 0) >= maxRange){
      hint = "head_down";
    } else if((face.headEulerAngleY ?? 0) >= maxRange){
      hint = Platform.isAndroid ? "head_right" : "head_left";
    } else if((face.headEulerAngleY ?? 0) <= -1 * maxRange){
      hint = Platform.isAndroid ? "head_left" : "head_right";
    } else if((face.headEulerAngleZ ?? 0) <= -1 * maxRange || (face.headEulerAngleZ ?? 0) >= maxRange){
      hint = "head_straight";
    } else if((face.leftEyeOpenProbability ?? 0) <= 0.8 || (face.rightEyeOpenProbability ?? 0) <= 0.8){
      hint = "open_eyes";
    } else if((face.smilingProbability ?? 0) >= 0.10){
      hint = "no_smile";
    } else {
      hint = "straight";
      await sendReport(_utils.getTaskIdFromVariant(TaskVariantEnum.straight), DateTime.now()).then((value) {
        result += stepSize;
        idle = Idle.done;
      }).catchError((onError){
        Utils.onError(context, onError);
      });
    }
  }
}
