import 'dart:convert';
import 'dart:developer';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:soliq_id/data/model/create_session_model.dart';
import 'package:soliq_id/utils/assets.dart';
import 'package:soliq_id/utils/color_utils.dart';
import 'package:soliq_id/utils/custom_button.dart';
import 'package:soliq_id/utils/typography.dart';
import 'package:soliq_id/utils/utils.dart';
import 'package:soliq_id/views/camera_screen.dart';
import 'package:url_launcher/url_launcher_string.dart';

class FaceIdSignIn extends StatefulWidget {
  final String pinfl;
  final String fullName;

  const FaceIdSignIn({
    Key? key,
    required this.pinfl,
    required this.fullName,
  }) : super(key: key);

  @override
  State<FaceIdSignIn> createState() => _FaceIdSignInState();
}

class _FaceIdSignInState extends State<FaceIdSignIn> {
  String? res;
  String buttonTitle = '';
  bool _isLoading = false;
  bool _isComparing = false;

  @override
  void initState() {
    log(widget.fullName);
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    buttonTitle =  'check';
    return Scaffold(
      backgroundColor: AppColor.appColor,
      appBar: AppBar(
        backgroundColor: AppColor.appColor,
        automaticallyImplyLeading: false,
        title: const Text(
          "Face ID",
          style: TextStyle(
            fontSize: 16,
            fontWeight: FontWeight.w500,
            color: Colors.black,
          ),
        ),
        actions: [
          InkWell(
            borderRadius: BorderRadius.circular(50),
            onTap: () {
              showDialog(
                  barrierDismissible: true,
                  barrierColor: const Color(0x77000000),
                  context: context,
                  builder: (_) {
                    return AlertDialog(
                      title: const Text(
                       'question_and_suggestion',
                        style: TextStyle(
                            fontSize: 14,
                            fontWeight: FontWeight.w600,
                            fontFamily: "Montserrat"),
                        textAlign: TextAlign.center,
                      ),
                      content: InkWell(
                        onTap: () {
                          launchUrlString("tel://+998712022202");
                        },
                        child: const Padding(
                          padding: EdgeInsets.all(8.0),
                          child: Text(
                            "+998 71 202 32 82",
                            textAlign: TextAlign.center,
                            style: TextStyle(
                                fontSize: 16,
                                color: Color(0xff6B95FF),
                                fontWeight: FontWeight.w600,
                                fontFamily: "Montserrat"),
                          ),
                        ),
                      ),
                    );
                  });
            },
            child: const Padding(
              padding: EdgeInsets.symmetric(horizontal: 16.0),
              child:
                  CircleAvatar(radius: 16, child: Icon(Icons.phone, size: 18)),
            ),
          )
        ],
        centerTitle: true,
        elevation: 0,
      ),
      body: Stack(
        children: [
          Align(
            alignment: Alignment.topCenter,
            child: Column(
              children: [
                const SizedBox(
                  height: 8,
                ),
                SizedBox(
                  width: MediaQuery.of(context).size.width / 2,
                  height: MediaQuery.of(context).size.width / 2,
                  child: Container(
                      margin: const EdgeInsets.all(18),
                      child: res == null
                          ? Image.asset(Assets.face_icon)
                          : ClipOval(
                          child: FittedBox(
                              fit: BoxFit.fitWidth,
                              child: Image.memory(base64Decode(res!))))),
                ),
                const SizedBox(
                  height: 24,
                ),
                Text(
                  widget.fullName,
                  style: const TextStyle(
                    fontSize: 16,
                    fontWeight: FontWeight.w700,
                  ),
                ),
                const SizedBox(
                  height: 6,
                ),
                Text(
                  "${ 'pinfl'}: ${widget.pinfl}",
                  style: const TextStyle(
                    fontSize: 16,
                    fontWeight: FontWeight.w700,
                  ),
                ),
                const SizedBox(
                  height: 26,
                ),
                Container(
                  margin: const EdgeInsets.symmetric(horizontal: 28),
                  child: const Text( 'suggest_advice',
                      textAlign: TextAlign.start,
                      style: FTypography.h1TextBold),
                ),
                const SizedBox(
                  height: 18,
                ),
                const Padding(
                  padding:  EdgeInsets.symmetric(
                    horizontal: 24,
                  ),
                  child: Text(
                    "advice",
                    textAlign: TextAlign.start,
                    style: FTypography.h1Text1,
                  ),
                ),
                const SizedBox(
                  height: 18,
                ),
              ],
            ),
          ),
          Positioned(
            bottom: 0,
            left: 0,
            right: 0,
            child: Container(
              margin: const EdgeInsets.only(bottom: 25),
              child: Column(
                children: [
                  AppButton(
                    title: buttonTitle,
                    isLoading: _isLoading,
                    isDisabled: _isLoading,
                    onTap: () async {
                      var session = await createSession();
                      if (session != null) {
                        var res = await  Navigator.push(
                            context,
                            CupertinoPageRoute(
                                builder: (_) => CameraScreen(
                                  session: session,
                                ))) as Map<String, dynamic>;
                        setState(() {});
                        if (res != null ||
                            res.toString().toLowerCase() != "null") {
                          Map<String, dynamic> params = {
                            "deviceId": await Utils.getDeviceId(),
                            "pinfl": widget.pinfl,
                            "fio": widget.fullName
                          };
                          params.addAll(res);

                          setState(() {
                            _isLoading = true;
                          });

                          // _apiProvider
                          //     .checkFaceID(context, params, session)
                          //     .then((value) {
                          //   log("value: ${value.toJson()}");
                          //   buttonTitle = getTranslated(context, 'try_again');
                          //   Navigator.push(
                          //       context,
                          //       CupertinoPageRoute(
                          //           builder: (_) => MessageScreen(
                          //               success: value.success,
                          //               message: value.message ?? "")));
                          // }).catchError((onError) {
                          //   log("onError: $onError");
                          //   Utils.onError(context, onError);
                          // }).whenComplete(() {
                          //   setState(() {
                          //     _isLoading = false;
                          //   });
                          // });

                        }
                      }
                    },
                  ),
                ],
              ),
            ),
          ),
        ],
      ),
    );
  }

  Future<CreateSessionModel?> createSession() async {
    CreateSessionModel? result;
    Map<String, dynamic> params = {};
    params["device"] =  (await Utils.getDeviceDetails()).toString();
    params["pinfl"] = widget.pinfl;
    params["fio"] = widget.fullName;

    setState(() {
      _isLoading = true;
    });

    // await _apiProvider.createSession(context, params).then((value) async {
    //   result = value;
    // }).catchError((onError) {
    //   Utils.onError(context, onError);
    // }).whenComplete(() {
    //   setState(() {
    //     _isLoading = false;
    //   });
    // });
    return result;
  }
}
